/// <reference types="node" />
import { GumVideoCaptureOptions, VideoPixelFormatEnum } from './VideoSupport';
import { CallLinkState, CallLinkRestrictions, CallLinkRootKey } from './CallLinks';
export declare const callIdFromEra: (era: string) => CallId;
export declare function callIdFromRingId(ringId: bigint): CallId;
declare class Config {
    field_trials: Record<string, string> | undefined;
}
type GroupId = Buffer;
type GroupCallUserId = Buffer;
export interface PeekDeviceInfo {
    demuxId: number;
    userId?: GroupCallUserId;
}
export interface Reaction {
    demuxId: number;
    value: string;
}
/** type returned by Rust */
export interface RawPeekInfo {
    devices: Array<PeekDeviceInfo>;
    creator?: GroupCallUserId;
    eraId?: string;
    maxDevices?: number;
    /** @deprecated Use {@link #deviceCountIncludingPendingDevices} and {@link #deviceCountExcludingPendingDevices} as appropriate */
    deviceCount: number;
    deviceCountIncludingPendingDevices: number;
    deviceCountExcludingPendingDevices: number;
    pendingUsers: Array<GroupCallUserId>;
    callLinkState?: RawCallLinkState;
}
/** type derived from RawPeekInfo */
export interface PeekInfo {
    devices: Array<PeekDeviceInfo>;
    creator?: GroupCallUserId;
    eraId?: string;
    maxDevices?: number;
    /** @deprecated Use {@link #deviceCountIncludingPendingDevices} and {@link #deviceCountExcludingPendingDevices} as appropriate */
    deviceCount: number;
    deviceCountIncludingPendingDevices: number;
    deviceCountExcludingPendingDevices: number;
    pendingUsers: Array<GroupCallUserId>;
    callLinkState?: CallLinkState;
}
export declare enum PeekStatusCodes {
    EXPIRED_CALL_LINK = 703,
    INVALID_CALL_LINK = 704
}
declare enum NetworkAdapterType {
    Unknown = 0,
    Ethernet = 1,
    Wifi = 2,
    Cellular = 4,
    Vpn = 8,
    Loopback = 16,
    Default = 32,
    Cellular2G = 64,
    Cellular3G = 128,
    Cellular4G = 256,
    Cellular5G = 512
}
export declare class NetworkRoute {
    localAdapterType: NetworkAdapterType;
    constructor();
}
export type RawAudioLevel = number;
export type NormalizedAudioLevel = number;
export declare class ReceivedAudioLevel {
    demuxId: number;
    level: RawAudioLevel;
    constructor(demuxId: number, level: RawAudioLevel);
}
interface RawCallLinkState {
    name: string;
    rawRestrictions: number;
    revoked: boolean;
    expiration: Date;
}
export type EmptyObj = Record<PropertyKey, never>;
export type HttpResult<T> = {
    success: true;
    value: T;
} | {
    success: false;
    errorStatusCode: number;
};
export declare class RingRTCType {
    private readonly callManager;
    private _call;
    private _groupCallByClientId;
    private _peekRequests;
    private _callLinkRequests;
    private _emptyRequests;
    private _callInfoByCallId;
    private getCallInfoKey;
    handleOutgoingSignaling: ((remoteUserId: UserId, message: CallingMessage) => Promise<boolean>) | null;
    handleIncomingCall: ((call: Call) => Promise<boolean>) | null;
    handleStartCall: ((call: Call) => Promise<boolean>) | null;
    handleAutoEndedIncomingCallRequest: ((callId: CallId, remoteUserId: UserId, reason: CallEndedReason, ageSec: number, wasVideoCall: boolean, receivedAtCounter: number | undefined) => void) | null;
    handleLogMessage: ((level: CallLogLevel, fileName: string, line: number, message: string) => void) | null;
    handleSendHttpRequest: ((requestId: number, url: string, method: HttpMethod, headers: {
        [name: string]: string;
    }, body: Buffer | undefined) => void) | null;
    handleSendCallMessage: ((recipientUuid: Buffer, message: Buffer, urgency: CallMessageUrgency) => void) | null;
    handleSendCallMessageToGroup: ((groupId: Buffer, message: Buffer, urgency: CallMessageUrgency, overrideRecipients: Array<Buffer>) => void) | null;
    handleGroupCallRingUpdate: ((groupId: Buffer, ringId: bigint, sender: Buffer, update: RingUpdate) => void) | null;
    constructor();
    setConfig(config: Config): void;
    setSelfUuid(uuid: Buffer): void;
    startOutgoingCall(remoteUserId: UserId, isVideoCall: boolean, localDeviceId: DeviceId): Call;
    cancelGroupRing(groupId: GroupId, ringId: bigint, reason: RingCancelReason | null): void;
    onStartOutgoingCall(remoteUserId: UserId, callId: CallId): void;
    onStartIncomingCall(remoteUserId: UserId, callId: CallId, isVideoCall: boolean): void;
    proceed(callId: CallId, settings: CallSettings): void;
    onCallState(remoteUserId: UserId, state: CallState): void;
    onCallEnded(remoteUserId: UserId, callId: CallId, reason: CallEndedReason, ageSec: number): void;
    onRemoteVideoEnabled(remoteUserId: UserId, enabled: boolean): void;
    onRemoteSharingScreen(remoteUserId: UserId, enabled: boolean): void;
    onNetworkRouteChanged(remoteUserId: UserId, localNetworkAdapterType: NetworkAdapterType): void;
    onAudioLevels(remoteUserId: UserId, capturedLevel: RawAudioLevel, receivedLevel: RawAudioLevel): void;
    onLowBandwidthForVideo(remoteUserId: UserId, recovered: boolean): void;
    renderVideoFrame(width: number, height: number, buffer: Buffer): void;
    onSendOffer(remoteUserId: UserId, remoteDeviceId: DeviceId, callId: CallId, broadcast: boolean, offerType: OfferType, opaque: Buffer): void;
    onSendAnswer(remoteUserId: UserId, remoteDeviceId: DeviceId, callId: CallId, broadcast: boolean, opaque: Buffer): void;
    onSendIceCandidates(remoteUserId: UserId, remoteDeviceId: DeviceId, callId: CallId, broadcast: boolean, candidates: Array<Buffer>): void;
    onSendHangup(remoteUserId: UserId, remoteDeviceId: DeviceId, callId: CallId, broadcast: boolean, hangupType: HangupType, deviceId: DeviceId | null): void;
    onSendBusy(remoteUserId: UserId, remoteDeviceId: DeviceId, callId: CallId, broadcast: boolean): void;
    private sendSignaling;
    /**
     * Asynchronous request to get information about a call link.
     *
     * @param sfuUrl - the URL to use when accessing the SFU
     * @param authCredentialPresentation - a serialized CallLinkAuthCredentialPresentation
     * @param linkRootKey - the root key for the call link
     *
     * Expected failure codes include:
     * - 404: the room does not exist (or expired so long ago that it has been removed from the server)
     */
    readCallLink(sfuUrl: string, authCredentialPresentation: Buffer, linkRootKey: CallLinkRootKey): Promise<HttpResult<CallLinkState>>;
    /**
     * Asynchronous request to create a new call link.
     *
     * This request is idempotent; if it fails due to a network issue, it is safe to retry.
     *
     * @example
     * const linkKey = CallLinkRootKey.generate();
     * const adminPasskey = CallLinkRootKey.generateAdminPasskey();
     * const roomId = linkKey.deriveRoomId();
     * const credential = requestCreateCredentialFromChatServer(roomId); // using libsignal
     * const secretParams = CallLinkSecretParams.deriveFromRootKey(linkKey.bytes);
     * const credentialPresentation = credential.present(roomId, secretParams).serialize();
     * const serializedPublicParams = secretParams.getPublicParams().serialize();
     * const result = await RingRTC.createCallLink(sfuUrl, credentialPresentation, linkKey, adminPasskey, serializedPublicParams);
     * if (result.success) {
     *   const state = result.value;
     *   // In actuality you may not want to do this until the user clicks Done.
     *   saveToDatabase(linkKey.bytes, adminPasskey, state);
     *   syncToOtherDevices(linkKey.bytes, adminPasskey);
     * } else {
     *   switch (result.errorStatusCode) {
     *   case 409:
     *     // The room already exists (and isn't yours), i.e. you've hit a 1-in-a-billion conflict.
     *     // Fall through to kicking the user out to try again later.
     *   default:
     *     // Unexpected error, kick the user out for now.
     *   }
     * }
     *
     * @param sfuUrl - the URL to use when accessing the SFU
     * @param createCredentialPresentation - a serialized CreateCallLinkCredentialPresentation
     * @param linkRootKey - the root key for the call link
     * @param adminPasskey - the arbitrary passkey to use for the new room
     * @param callLinkPublicParams - the serialized CallLinkPublicParams for the new room
     */
    createCallLink(sfuUrl: string, createCredentialPresentation: Buffer, linkRootKey: CallLinkRootKey, adminPasskey: Buffer, callLinkPublicParams: Buffer): Promise<HttpResult<CallLinkState>>;
    /**
     * Asynchronous request to update a call link's name.
     *
     * Possible failure codes include:
     * - 401: the room does not exist (and this is the wrong API to create a new room)
     * - 403: the admin passkey is incorrect
     *
     * This request is idempotent; if it fails due to a network issue, it is safe to retry.
     *
     * @param sfuUrl - the URL to use when accessing the SFU
     * @param authCredentialPresentation - a serialized CallLinkAuthCredentialPresentation
     * @param linkRootKey - the root key for the call link
     * @param adminPasskey - the passkey specified when the link was created
     * @param newName - the new name to use
     */
    updateCallLinkName(sfuUrl: string, authCredentialPresentation: Buffer, linkRootKey: CallLinkRootKey, adminPasskey: Buffer, newName: string): Promise<HttpResult<CallLinkState>>;
    /**
     * Asynchronous request to update a call link's restrictions.
     *
     * Possible failure codes include:
     * - 401: the room does not exist (and this is the wrong API to create a new room)
     * - 403: the admin passkey is incorrect
     * - 409: the room is currently in use, so restrictions cannot be changed at the moment
     *
     * This request is idempotent; if it fails due to a network issue, it is safe to retry.
     *
     * @param sfuUrl - the URL to use when accessing the SFU
     * @param authCredentialPresentation - a serialized CallLinkAuthCredentialPresentation
     * @param linkRootKey - the root key for the call link
     * @param adminPasskey - the passkey specified when the link was created
     * @param restrictions - the new restrictions to use
     */
    updateCallLinkRestrictions(sfuUrl: string, authCredentialPresentation: Buffer, linkRootKey: CallLinkRootKey, adminPasskey: Buffer, restrictions: Exclude<CallLinkRestrictions, CallLinkRestrictions.Unknown>): Promise<HttpResult<CallLinkState>>;
    /**
     * Asynchronous request to delete a call link.
     *
     * Possible failure codes include:
     * - 403: the admin passkey is incorrect
     * - 409: conflict - there is an ongoing call for this room.
     *
     * This request is idempotent; if it fails due to a network issue, it is safe to retry.
     *
     * @param sfuUrl - the URL to use when accessing the SFU
     * @param authCredentialPresentation - a serialized CallLinkAuthCredentialPresentation
     * @param linkRootKey - the root key for the call link
     * @param adminPasskey - the passkey specified when the link was created
     */
    deleteCallLink(sfuUrl: string, authCredentialPresentation: Buffer, linkRootKey: CallLinkRootKey, adminPasskey: Buffer): Promise<HttpResult<undefined>>;
    receivedHttpResponse(requestId: number, status: number, body: Buffer): void;
    httpRequestFailed(requestId: number, debugInfo: string | undefined): void;
    getGroupCall(groupId: Buffer, sfuUrl: string, hkdfExtraInfo: Buffer, audioLevelsIntervalMillis: number | undefined, observer: GroupCallObserver): GroupCall | undefined;
    getCallLinkCall(sfuUrl: string, authCredentialPresentation: Buffer, rootKey: CallLinkRootKey, adminPasskey: Buffer | undefined, hkdfExtraInfo: Buffer, audioLevelsIntervalMillis: number | undefined, observer: GroupCallObserver): GroupCall | undefined;
    peekGroupCall(sfuUrl: string, membershipProof: Buffer, groupMembers: Array<GroupMemberInfo>): Promise<PeekInfo>;
    peekCallLinkCall(sfuUrl: string, authCredentialPresentation: Buffer, rootKey: CallLinkRootKey): Promise<HttpResult<PeekInfo>>;
    requestMembershipProof(clientId: GroupCallClientId): void;
    requestGroupMembers(clientId: GroupCallClientId): void;
    handleConnectionStateChanged(clientId: GroupCallClientId, connectionState: ConnectionState): void;
    handleJoinStateChanged(clientId: GroupCallClientId, joinState: JoinState, demuxId: number | undefined): void;
    handleNetworkRouteChanged(clientId: GroupCallClientId, localNetworkAdapterType: NetworkAdapterType): void;
    handleAudioLevels(clientId: GroupCallClientId, capturedLevel: RawAudioLevel, receivedLevels: Array<ReceivedAudioLevel>): void;
    handleLowBandwidthForVideo(clientId: GroupCallClientId, recovered: boolean): void;
    handleReactions(clientId: GroupCallClientId, reactions: Array<Reaction>): void;
    handleRaisedHands(clientId: GroupCallClientId, raisedHands: Array<number>): void;
    handleRemoteDevicesChanged(clientId: GroupCallClientId, remoteDeviceStates: Array<RemoteDeviceState>): void;
    handlePeekChanged(clientId: GroupCallClientId, rawInfo: RawPeekInfo): void;
    handlePeekResponse(requestId: number, statusCode: number, rawInfo: RawPeekInfo | undefined): void;
    handleCallLinkResponse(requestId: number, statusCode: number, state: RawCallLinkState | undefined): void;
    handleEmptyResponse(requestId: number, statusCode: number, state: EmptyObj | undefined): void;
    handleEnded(clientId: GroupCallClientId, reason: GroupCallEndReason): void;
    groupCallRingUpdate(groupId: GroupId, ringId: bigint, sender: GroupCallUserId, state: RingUpdate): void;
    onLogMessage(level: number, fileName: string, line: number, message: string): void;
    logError(message: string): void;
    logWarn(message: string): void;
    logInfo(message: string): void;
    handleCallingMessage(remoteUserId: UserId, remoteUuid: Buffer | null, remoteDeviceId: DeviceId, localDeviceId: DeviceId, messageAgeSec: number, messageReceivedAtCounter: number, message: CallingMessage, senderIdentityKey: Buffer, receiverIdentityKey: Buffer): void;
    sendHttpRequest(requestId: number, url: string, method: HttpMethod, headers: {
        [name: string]: string;
    }, body: Buffer | undefined): void;
    sendCallMessage(recipientUuid: Buffer, message: Buffer, urgency: CallMessageUrgency): void;
    sendCallMessageToGroup(groupId: Buffer, message: Buffer, urgency: CallMessageUrgency, overrideRecipients: Array<Buffer>): void;
    get call(): Call | null;
    getCall(callId: CallId): Call | null;
    accept(callId: CallId, asVideoCall: boolean): void;
    decline(callId: CallId): void;
    ignore(callId: CallId): void;
    hangup(callId: CallId): void;
    setOutgoingAudio(callId: CallId, enabled: boolean): void;
    setOutgoingVideo(callId: CallId, enabled: boolean): void;
    setOutgoingVideoIsScreenShare(callId: CallId, isScreenShare: boolean): void;
    setVideoCapturer(callId: CallId, capturer: VideoCapturer | null): void;
    setVideoRenderer(callId: CallId, renderer: VideoRenderer | null): void;
    getAudioInputs(): Array<AudioDevice>;
    setAudioInput(index: number): void;
    getAudioOutputs(): Array<AudioDevice>;
    setAudioOutput(index: number): void;
}
export interface CallSettings {
    iceServers: Array<IceServer>;
    hideIp: boolean;
    dataMode: DataMode;
    audioLevelsIntervalMillis?: number;
}
interface IceServer {
    username?: string;
    password?: string;
    /** Provide hostname when urls contain IP addresses instead of hostname */
    hostname?: string;
    urls: Array<string>;
}
export interface AudioDevice {
    name: string;
    index: number;
    uniqueId: string;
    i18nKey?: string;
}
export interface VideoCapturer {
    enableCapture(): void;
    enableCaptureAndSend(call: Call, captureOptions?: GumVideoCaptureOptions): void;
    disable(): void;
}
export interface VideoRenderer {
    enable(call: Call): void;
    disable(): void;
}
export declare class Call {
    private readonly _callManager;
    private readonly _remoteUserId;
    callId: CallId;
    private readonly _isIncoming;
    private readonly _isVideoCall;
    private _state;
    private _outgoingAudioEnabled;
    private _outgoingVideoEnabled;
    private _outgoingVideoIsScreenShare;
    private _remoteVideoEnabled;
    outgoingAudioLevel: NormalizedAudioLevel;
    remoteAudioLevel: NormalizedAudioLevel;
    remoteSharingScreen: boolean;
    networkRoute: NetworkRoute;
    private _videoCapturer;
    private _videoRenderer;
    endedReason?: CallEndedReason;
    handleStateChanged?: () => void;
    handleRemoteVideoEnabled?: () => void;
    handleRemoteSharingScreen?: () => void;
    handleNetworkRouteChanged?: () => void;
    handleAudioLevels?: () => void;
    /**
     * Notification of low upload bandwidth for sending video.
     *
     * When this is first called, recovered will be false. The second call (if
     * any) will have recovered set to true and will be called when the upload
     * bandwidth is high enough to send video.
     *
     * @param recovered - whether there is enough bandwidth to send video reliably
     */
    handleLowBandwidthForVideo?: (recovered: boolean) => void;
    renderVideoFrame?: (width: number, height: number, buffer: Buffer) => void;
    constructor(callManager: CallManager, remoteUserId: UserId, callId: CallId, isIncoming: boolean, isVideoCall: boolean, state: CallState);
    get remoteUserId(): UserId;
    get isIncoming(): boolean;
    get isVideoCall(): boolean;
    get state(): CallState;
    set state(state: CallState);
    setCallEnded(): void;
    set videoCapturer(capturer: VideoCapturer | null);
    set videoRenderer(renderer: VideoRenderer | null);
    accept(): void;
    decline(): void;
    ignore(): void;
    hangup(): void;
    get outgoingAudioEnabled(): boolean;
    set outgoingAudioEnabled(enabled: boolean);
    get outgoingVideoEnabled(): boolean;
    set outgoingVideoEnabled(enabled: boolean);
    set outgoingVideoIsScreenShare(isScreenShare: boolean);
    get remoteVideoEnabled(): boolean;
    set remoteVideoEnabled(enabled: boolean);
    sendVideoFrame(width: number, height: number, format: VideoPixelFormatEnum, buffer: Buffer): void;
    receiveVideoFrame(buffer: Buffer, maxWidth: number, maxHeight: number): [number, number] | undefined;
    private enableOrDisableCapturer;
    private setOutgoingVideoEnabled;
    updateDataMode(dataMode: DataMode): void;
    private enableOrDisableRenderer;
}
export type GroupCallClientId = number;
export declare enum ConnectionState {
    NotConnected = 0,
    Connecting = 1,
    Connected = 2,
    Reconnecting = 3
}
export declare enum JoinState {
    NotJoined = 0,
    Joining = 1,
    Pending = 2,
    Joined = 3
}
export declare enum GroupCallEndReason {
    DeviceExplicitlyDisconnected = 0,
    ServerExplicitlyDisconnected = 1,
    DeniedRequestToJoinCall = 2,
    RemovedFromCall = 3,
    CallManagerIsBusy = 4,
    SfuClientFailedToJoin = 5,
    FailedToCreatePeerConnectionFactory = 6,
    FailedToNegotiateSrtpKeys = 7,
    FailedToCreatePeerConnection = 8,
    FailedToStartPeerConnection = 9,
    FailedToUpdatePeerConnection = 10,
    FailedToSetMaxSendBitrate = 11,
    IceFailedWhileConnecting = 12,
    IceFailedAfterConnected = 13,
    ServerChangedDemuxId = 14,
    HasMaxDevices = 15
}
export declare enum CallMessageUrgency {
    Droppable = 0,
    HandleImmediately = 1
}
export declare enum RingUpdate {
    Requested = 0,
    ExpiredRequest = 1,
    AcceptedOnAnotherDevice = 2,
    DeclinedOnAnotherDevice = 3,
    BusyLocally = 4,
    BusyOnAnotherDevice = 5,
    CancelledByRinger = 6
}
export declare enum HttpMethod {
    Get = 0,
    Put = 1,
    Post = 2,
    Delete = 3
}
export declare class LocalDeviceState {
    connectionState: ConnectionState;
    joinState: JoinState;
    demuxId?: number;
    audioMuted: boolean;
    videoMuted: boolean;
    audioLevel: NormalizedAudioLevel;
    presenting: boolean;
    sharingScreen: boolean;
    networkRoute: NetworkRoute;
    constructor();
}
export declare class RemoteDeviceState {
    demuxId: number;
    userId: Buffer;
    mediaKeysReceived: boolean;
    audioMuted: boolean | undefined;
    videoMuted: boolean | undefined;
    audioLevel: NormalizedAudioLevel;
    presenting: boolean | undefined;
    sharingScreen: boolean | undefined;
    videoAspectRatio: number | undefined;
    addedTime: string;
    speakerTime: string;
    forwardingVideo: boolean | undefined;
    isHigherResolutionPending: boolean;
    constructor(demuxId: number, userId: Buffer, addedTime: string, speakerTime: string, mediaKeysReceived: boolean);
}
export declare class GroupMemberInfo {
    userId: Buffer;
    userIdCipherText: Buffer;
    constructor(userId: Buffer, userIdCipherText: Buffer);
}
export declare class VideoRequest {
    demuxId: number;
    width: number;
    height: number;
    framerate: number | undefined;
    constructor(demuxId: number, width: number, height: number, framerate: number | undefined);
}
export declare enum GroupCallKind {
    SignalGroup = 0,
    CallLink = 1
}
export interface GroupCallObserver {
    requestMembershipProof(groupCall: GroupCall): void;
    requestGroupMembers(groupCall: GroupCall): void;
    onLocalDeviceStateChanged(groupCall: GroupCall): void;
    onRemoteDeviceStatesChanged(groupCall: GroupCall): void;
    onAudioLevels(groupCall: GroupCall): void;
    onLowBandwidthForVideo(groupCall: GroupCall, recovered: boolean): void;
    onReactions(groupCall: GroupCall, reactions: Array<Reaction>): void;
    onRaisedHands(groupCall: GroupCall, raisedHands: Array<number>): void;
    onPeekChanged(groupCall: GroupCall): void;
    onEnded(groupCall: GroupCall, reason: GroupCallEndReason): void;
}
export declare class GroupCall {
    private readonly _kind;
    private readonly _callManager;
    private readonly _observer;
    private readonly _clientId;
    get clientId(): GroupCallClientId;
    private _localDeviceState;
    private _remoteDeviceStates;
    private _peekInfo;
    constructor(kind: GroupCallKind, callManager: CallManager, observer: GroupCallObserver, clientId: GroupCallClientId);
    getKind(): GroupCallKind;
    connect(): void;
    join(): void;
    leave(): void;
    disconnect(): void;
    getLocalDeviceState(): LocalDeviceState;
    getRemoteDeviceStates(): Array<RemoteDeviceState> | undefined;
    getPeekInfo(): PeekInfo | undefined;
    setOutgoingAudioMuted(muted: boolean): void;
    react(value: string): void;
    raiseHand(raise: boolean): void;
    setOutgoingVideoMuted(muted: boolean): void;
    setPresenting(presenting: boolean): void;
    setOutgoingVideoIsScreenShare(isScreenShare: boolean): void;
    ringAll(): void;
    resendMediaKeys(): void;
    setDataMode(dataMode: DataMode): void;
    requestVideo(resolutions: Array<VideoRequest>, activeSpeakerHeight: number): void;
    approveUser(otherUserId: Buffer): void;
    denyUser(otherUserId: Buffer): void;
    removeClient(otherClientDemuxId: number): void;
    blockClient(otherClientDemuxId: number): void;
    setGroupMembers(members: Array<GroupMemberInfo>): void;
    setMembershipProof(proof: Buffer): void;
    requestMembershipProof(): void;
    requestGroupMembers(): void;
    handleConnectionStateChanged(connectionState: ConnectionState): void;
    handleJoinStateChanged(joinState: JoinState, demuxId: number | undefined): void;
    handleNetworkRouteChanged(localNetworkAdapterType: NetworkAdapterType): void;
    handleAudioLevels(capturedLevel: RawAudioLevel, receivedLevels: Array<ReceivedAudioLevel>): void;
    handleLowBandwidthForVideo(recovered: boolean): void;
    handleReactions(reactions: Array<Reaction>): void;
    handleRaisedHands(raisedHands: Array<number>): void;
    handleRemoteDevicesChanged(remoteDeviceStates: Array<RemoteDeviceState>): void;
    handlePeekChanged(info: PeekInfo): void;
    handleEnded(reason: GroupCallEndReason): void;
    sendVideoFrame(width: number, height: number, format: VideoPixelFormatEnum, buffer: Buffer): void;
    getVideoSource(remoteDemuxId: number): GroupCallVideoFrameSource;
    setRemoteAspectRatio(remoteDemuxId: number, aspectRatio: number): void;
}
declare class GroupCallVideoFrameSource {
    private readonly _callManager;
    private readonly _groupCall;
    private readonly _remoteDemuxId;
    constructor(callManager: CallManager, groupCall: GroupCall, remoteDemuxId: number);
    receiveVideoFrame(buffer: Buffer, maxWidth: number, maxHeight: number): [number, number] | undefined;
}
type ProtobufBuffer = Buffer | {
    toArrayBuffer: () => ArrayBuffer;
};
export type UserId = string;
export type DeviceId = number;
export type CallId = {
    high: number;
    low: number;
    unsigned: boolean;
};
export declare class CallingMessage {
    offer?: OfferMessage;
    answer?: AnswerMessage;
    iceCandidates?: Array<IceCandidateMessage>;
    busy?: BusyMessage;
    hangup?: HangupMessage;
    opaque?: OpaqueMessage;
    destinationDeviceId?: DeviceId;
}
export declare class OfferMessage {
    callId: CallId;
    type: OfferType;
    opaque: ProtobufBuffer;
    constructor(callId: CallId, type: OfferType, opaque: ProtobufBuffer);
}
export declare enum OfferType {
    AudioCall = 0,
    VideoCall = 1
}
export declare class AnswerMessage {
    callId: CallId;
    opaque: ProtobufBuffer;
    constructor(callId: CallId, opaque: ProtobufBuffer);
}
export declare class IceCandidateMessage {
    callId: CallId;
    opaque: ProtobufBuffer;
    constructor(callId: CallId, opaque: ProtobufBuffer);
}
export declare class BusyMessage {
    callId: CallId;
    constructor(callId: CallId);
}
export declare class HangupMessage {
    callId: CallId;
    type: HangupType;
    deviceId: DeviceId;
    constructor(callId: CallId, type: HangupType, deviceId: DeviceId);
}
export declare class OpaqueMessage {
    data?: ProtobufBuffer;
}
export declare enum HangupType {
    Normal = 0,
    Accepted = 1,
    Declined = 2,
    Busy = 3,
    NeedPermission = 4
}
export declare enum DataMode {
    Low = 0,
    Normal = 1
}
export declare enum RingCancelReason {
    DeclinedByUser = 0,
    Busy = 1
}
export interface CallManager {
    setConfig(config: Config): void;
    setSelfUuid(uuid: Buffer): void;
    createOutgoingCall(remoteUserId: UserId, isVideoCall: boolean, localDeviceId: DeviceId): CallId;
    proceed(callId: CallId, iceServers: Array<IceServer>, hideIp: boolean, dataMode: DataMode, audioLevelsIntervalMillis: number): void;
    accept(callId: CallId): void;
    ignore(callId: CallId): void;
    hangup(): void;
    cancelGroupRing(groupId: GroupId, ringId: string, reason: RingCancelReason | null): void;
    signalingMessageSent(callId: CallId): void;
    signalingMessageSendFailed(callId: CallId): void;
    setOutgoingAudioEnabled(enabled: boolean): void;
    setOutgoingVideoEnabled(enabled: boolean): void;
    setOutgoingVideoIsScreenShare(enabled: boolean): void;
    updateDataMode(dataMode: DataMode): void;
    sendVideoFrame(width: number, height: number, format: VideoPixelFormatEnum, buffer: Buffer): void;
    receiveVideoFrame(buffer: Buffer, maxWidth: number, maxHeight: number): [number, number] | undefined;
    receivedOffer(remoteUserId: UserId, remoteDeviceId: DeviceId, localDeviceId: DeviceId, messageAgeSec: number, callId: CallId, offerType: OfferType, opaque: Buffer, senderIdentityKey: Buffer, receiverIdentityKey: Buffer): void;
    receivedAnswer(remoteUserId: UserId, remoteDeviceId: DeviceId, callId: CallId, opaque: Buffer, senderIdentityKey: Buffer, receiverIdentityKey: Buffer): void;
    receivedIceCandidates(remoteUserId: UserId, remoteDeviceId: DeviceId, callId: CallId, candidates: Array<Buffer>): void;
    receivedHangup(remoteUserId: UserId, remoteDeviceId: DeviceId, callId: CallId, hangupType: HangupType, hangupDeviceId: DeviceId | null): void;
    receivedBusy(remoteUserId: UserId, remoteDeviceId: DeviceId, callId: CallId): void;
    receivedCallMessage(remoteUserId: Buffer, remoteDeviceId: DeviceId, localDeviceId: DeviceId, data: Buffer, messageAgeSec: number): void;
    receivedHttpResponse(requestId: number, status: number, body: Buffer): void;
    httpRequestFailed(requestId: number, debugInfo: string | undefined): void;
    createGroupCallClient(groupId: Buffer, sfuUrl: string, hkdfExtraInfo: Buffer, audioLevelsIntervalMillis: number): GroupCallClientId;
    createCallLinkCallClient(sfuUrl: string, authCredentialPresentation: Buffer, linkRootKey: Buffer, adminPasskey: Buffer | undefined, hkdfExtraInfo: Buffer, audioLevelsIntervalMillis: number): GroupCallClientId;
    deleteGroupCallClient(clientId: GroupCallClientId): void;
    connect(clientId: GroupCallClientId): void;
    join(clientId: GroupCallClientId): void;
    leave(clientId: GroupCallClientId): void;
    disconnect(clientId: GroupCallClientId): void;
    setOutgoingAudioMuted(clientId: GroupCallClientId, muted: boolean): void;
    setOutgoingVideoMuted(clientId: GroupCallClientId, muted: boolean): void;
    setPresenting(clientId: GroupCallClientId, presenting: boolean): void;
    setOutgoingGroupCallVideoIsScreenShare(clientId: GroupCallClientId, isScreenShare: boolean): void;
    groupRing(clientId: GroupCallClientId, recipient: Buffer | undefined): void;
    groupReact(clientId: GroupCallClientId, value: string): void;
    groupRaiseHand(clientId: GroupCallClientId, raise: boolean): void;
    resendMediaKeys(clientId: GroupCallClientId): void;
    setDataMode(clientId: GroupCallClientId, dataMode: DataMode): void;
    requestVideo(clientId: GroupCallClientId, resolutions: Array<VideoRequest>, activeSpeakerHeight: number): void;
    approveUser(clientId: GroupCallClientId, otherUserId: Buffer): void;
    denyUser(clientId: GroupCallClientId, otherUserId: Buffer): void;
    removeClient(clientId: GroupCallClientId, otherClientDemuxId: number): void;
    blockClient(clientId: GroupCallClientId, otherClientDemuxId: number): void;
    setGroupMembers(clientId: GroupCallClientId, members: Array<GroupMemberInfo>): void;
    setMembershipProof(clientId: GroupCallClientId, proof: Buffer): void;
    receiveGroupCallVideoFrame(clientId: GroupCallClientId, remoteDemuxId: number, buffer: Buffer, maxWidth: number, maxHeight: number): [number, number] | undefined;
    readCallLink(requestId: number, sfuUrl: string, authCredentialPresentation: Buffer, linkRootKey: Buffer): void;
    createCallLink(requestId: number, sfuUrl: string, createCredentialPresentation: Buffer, linkRootKey: Buffer, adminPasskey: Buffer, callLinkPublicParams: Buffer): void;
    updateCallLink(requestId: number, sfuUrl: string, authCredentialPresentation: Buffer, linkRootKey: Buffer, adminPasskey: Buffer, newName: string | undefined, newRestrictions: number | undefined, newRevoked: boolean | undefined): void;
    deleteCallLink(requestId: number, sfuUrl: string, authCredentialPresentation: Buffer, linkRootKey: Buffer, adminPasskey: Buffer): void;
    peekGroupCall(requestId: number, sfu_url: string, membership_proof: Buffer, group_members: Array<GroupMemberInfo>): void;
    peekCallLinkCall(requestId: number, sfuUrl: string, authCredentialPresentation: Buffer, linkRootKey: Buffer): void;
    getAudioInputs(): Array<AudioDevice>;
    setAudioInput(index: number): void;
    getAudioOutputs(): Array<AudioDevice>;
    setAudioOutput(index: number): void;
}
export interface CallManagerCallbacks {
    onStartOutgoingCall(remoteUserId: UserId, callId: CallId): void;
    onStartIncomingCall(remoteUserId: UserId, callId: CallId, isVideoCall: boolean): void;
    onCallState(remoteUserId: UserId, state: CallState): void;
    onCallEnded(remoteUserId: UserId, callId: CallId, endedReason: CallEndedReason, ageSec: number): void;
    onRemoteVideoEnabled(remoteUserId: UserId, enabled: boolean): void;
    onRemoteSharingScreen(remoteUserId: UserId, enabled: boolean): void;
    onSendOffer(remoteUserId: UserId, remoteDeviceId: DeviceId, callId: CallId, broadcast: boolean, mediaType: number, opaque: Buffer): void;
    onSendAnswer(remoteUserId: UserId, remoteDeviceId: DeviceId, callId: CallId, broadcast: boolean, opaque: Buffer): void;
    onSendIceCandidates(remoteUserId: UserId, remoteDeviceId: DeviceId, callId: CallId, broadcast: boolean, candidates: Array<Buffer>): void;
    onSendHangup(remoteUserId: UserId, remoteDeviceId: DeviceId, callId: CallId, broadcast: boolean, HangupType: HangupType, hangupDeviceId: DeviceId | null): void;
    onSendBusy(remoteUserId: UserId, remoteDeviceId: DeviceId, callId: CallId, broadcast: boolean): void;
    sendCallMessage(recipientUuid: Buffer, message: Buffer, urgency: CallMessageUrgency): void;
    sendCallMessageToGroup(groupId: Buffer, message: Buffer, urgency: CallMessageUrgency, overrideRecipients: Array<Buffer>): void;
    sendHttpRequest(requestId: number, url: string, method: HttpMethod, headers: {
        [name: string]: string;
    }, body: Buffer | undefined): void;
    requestMembershipProof(clientId: GroupCallClientId): void;
    requestGroupMembers(clientId: GroupCallClientId): void;
    handleConnectionStateChanged(clientId: GroupCallClientId, connectionState: ConnectionState): void;
    handleJoinStateChanged(clientId: GroupCallClientId, joinState: JoinState, demuxId: number | undefined): void;
    handleRemoteDevicesChanged(clientId: GroupCallClientId, remoteDeviceStates: Array<RemoteDeviceState>): void;
    handlePeekChanged(clientId: GroupCallClientId, rawInfo: RawPeekInfo): void;
    handlePeekResponse(requestId: number, statusCode: number, rawInfo: RawPeekInfo | undefined): void;
    handleEnded(clientId: GroupCallClientId, reason: GroupCallEndReason): void;
    onLogMessage(level: number, fileName: string, line: number, message: string): void;
}
export declare enum CallState {
    Prering = "idle",
    Ringing = "ringing",
    Accepted = "connected",
    Reconnecting = "connecting",
    Ended = "ended"
}
export declare enum CallEndedReason {
    LocalHangup = "LocalHangup",
    RemoteHangup = "RemoteHangup",
    RemoteHangupNeedPermission = "RemoteHangupNeedPermission",
    Declined = "Declined",
    Busy = "Busy",
    Glare = "Glare",
    ReCall = "ReCall",
    ReceivedOfferExpired = "ReceivedOfferExpired",
    ReceivedOfferWhileActive = "ReceivedOfferWhileActive",
    ReceivedOfferWithGlare = "ReceivedOfferWithGlare",
    SignalingFailure = "SignalingFailure",
    GlareFailure = "GlareFailure",
    ConnectionFailure = "ConnectionFailure",
    InternalFailure = "InternalFailure",
    Timeout = "Timeout",
    AcceptedOnAnotherDevice = "AcceptedOnAnotherDevice",
    DeclinedOnAnotherDevice = "DeclinedOnAnotherDevice",
    BusyOnAnotherDevice = "BusyOnAnotherDevice"
}
export declare enum CallLogLevel {
    Off = 0,
    Error = 1,
    Warn = 2,
    Info = 3,
    Debug = 4,
    Trace = 5
}
export {};
